/*
   mkbd.ino
   Copyright (c) 2021 @triring All Rights Reserved.
   Created: 2021年3月14日 日曜
   Author: @triring
   home-brew midi keyboard
   自作キーボードキット plum47のファームウェアを書き換えて、MIDIキーボードにする。
   https://yushakobo.jp/shop/consign_plum47/
   https://github.com/takagi/plum47_buildguide
   https://github.com/takagi/qmk_firmware/tree/plum47/keyboards
   DTMのピアノロールを模した配列とし、1行12個のキーに白鍵と黒鍵をそのまま割り当てた。
   https://ja.wikipedia.org/wiki/%E3%83%94%E3%82%A2%E3%83%8E%E3%83%AD%E3%83%BC%E3%83%AB#DTM%E3%81%AB%E3%81%8A%E3%81%91%E3%82%8B%E3%83%94%E3%82%A2%E3%83%8E%E3%83%AD%E3%83%BC%E3%83%AB
  　　最上段の行より、国際式5オクターブ、国際式4オクターブ、国際式3オクターブとし、
  　　最下段の行はオクターブシフト用に割り当てた。
  、、中央のスペースは、オクターブシフトなし。
   中央のスペースから、右へ1つづ、1，2オクターブ高音へシフト
   中央のスペースから、左へ1つづ、1，2オクターブ低音へシフト
  　　両端の3個は、未割り当て
  以下のサイトを起動すれば、何も準備しなくても、テストができる。
  https://webaudiodemos.appspot.com/midi-synth/
*/

/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * @triring wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return @triring
 * ----------------------------------------------------------------------------
[ * "THE BEER-WARE LICENSE" (Revision 42)]: https://people.freebsd.org/~phk/
 */

#include "MIDIUSB.h"

const int rowNum =  4;
const int colNum = 12;

// ピンの割当 for plum47
const int rowPin[rowNum] = { 2, 3, 4, 5 };
const int colPin[colNum] = { A3, A2, A1, A0, 15, 14, 16, 10, 9, 8, 7, 6};

// MIDI 音階表
const byte noteKeyMap[rowNum][colNum]  = {
  //  C    C#   D    D#   E    F    F#   G    G#   A    A#   B
  {  72,  73,  74,  75,  76,  77,  78,  79,  80,  81,  82,  83 },
  {  60,  61,  62,  63,  64,  65,  66,  67,  68,  69,  70,  71 },
  {  48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59 },
  {   0,   0, -36, -24, -12,   0,   0,  12,  24,  36,   0,   0 }
};

// 押されたキーの情報を管理するための配列
bool currentState[rowNum][colNum];
bool beforeState[rowNum][colNum];
byte currentNote[rowNum][colNum];

int row, col;
byte note = 0;
byte oct_shift = 0;

void setup() {
  // Serial.begin(115200);
  Serial.begin(31250);
  // 行単位の制御ピンの初期化
  for ( row = 0; row < rowNum; row++) {
    pinMode(rowPin[row], OUTPUT);
    delay(10);
    digitalWrite(rowPin[row], HIGH);
  }

  // 列情報読込みピンの初期化
  for ( col = 0; col < colNum; col++) {
    pinMode(colPin[col], INPUT_PULLUP);
  }
  // 読み取ったキーの情報を記録しておく配列を初期化する。
  for (row = 0; row < rowNum; row++) {
    for ( col = 0; col < colNum; col++) {
      currentState[row][col] = HIGH;
      beforeState[row][col] = HIGH;
      currentNote[rowNum][colNum] = 0;
    }
  }
}

void loop() {
  // コントロールキーの読み取り
  row = rowNum - 1;
  digitalWrite( rowPin[row], LOW ); // 最下段を選択
  for ( col = 0; col < colNum; col++) {
    if (LOW == digitalRead(colPin[col])) {
      oct_shift = noteKeyMap[row][col];
    }
  }
  digitalWrite( rowPin[row], HIGH );

  // 音階キーの読取りと発音
  for (row = 0; row < rowNum - 1; row++) {
    digitalWrite( rowPin[row], LOW );
    for ( col = 0; col < colNum; col++) {
      currentState[row][col] = digitalRead(colPin[col]);
      if ( currentState[row][col] != beforeState[row][col] ) {
        Serial.print("key(");
        Serial.print(row);
        Serial.print(",");
        Serial.print(col);
        Serial.print(")");
        if ( currentState[row][col] == LOW) {
          note = noteKeyMap[row][col] + oct_shift;
          noteOn(1, note, 64);   // Channel 0, note, normal velocity
          currentNote[row][col] = note;
          MidiUSB.flush();
          Serial.println(" Push!");
          //  Keyboard.press( keyMap[row][col] );
        } else {
          note = currentNote[row][col];
          noteOff(1, note, 64);   // Channel 0, note, normal velocity
          MidiUSB.flush();
          Serial.println(" Release!");
          //  Keyboard.release( keyMap[row][col] );
        }
        beforeState[row][col] = currentState[row][col];
      }
    }
    digitalWrite( rowPin[row], HIGH );
  }
}

void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
}
// First parameter is the event type (0x09 = note on, 0x08 = note off).
// Second parameter is note-on/note-off, combined with the channel.
// Channel can be anything between 0-15. Typically reported to the user as 1-16.
// Third parameter is the note number (48 = middle C).
// Fourth parameter is the velocity (64 = normal, 127 = fastest).
void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
}

// First parameter is the event type (0x0B = control change).
// Second parameter is the event type, combined with the channel.
// Third parameter is the control number number (0-119).
// Fourth parameter is the control value (0-127).

void controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = {0x0B, 0xB0 | channel, control, value};
  MidiUSB.sendMIDI(event);
}
